"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var Logger = /** @class */ (function () {
    function Logger() {
        var _this = this;
        this.LOG_TYPE = {
            INFO: 'info',
            DEBUG: 'debug',
            ERROR: 'error',
            WARN: 'warn'
        };
        this._dateString = function (_a) {
            var _b = _a === void 0 ? {} : _a, _c = _b.time, time = _c === void 0 ? true : _c;
            var d = new Date();
            var year = d.getFullYear();
            var month = ("0".concat((d.getMonth() + 1))).slice(-2);
            var date = ("0".concat(d.getDate())).slice(-2);
            var hours = ("0".concat(d.getHours())).slice(-2);
            var minutes = ("0".concat(d.getMinutes())).slice(-2);
            var seconds = ("0".concat(d.getSeconds())).slice(-2);
            if (time)
                return "".concat(year, "-").concat(month, "-").concat(date, " ").concat(hours, ":").concat(minutes, ":").concat(seconds);
            return "".concat(year, "-").concat(month, "-").concat(date);
        };
        this._createFolder = function () {
            try {
                fs_1.default.accessSync(_this.LOG_FOLDER, fs_1.default.constants.F_OK);
            }
            catch (err) {
                fs_1.default.mkdirSync(_this.LOG_FOLDER, {
                    recursive: true
                });
            }
            return _this;
        };
        this._createLogFiles = function (_a) {
            var _b = _a.type, type = _b === void 0 ? 'error' : _b, _c = _a.message, message = _c === void 0 ? '' : _c, _d = _a.stack, stack = _d === void 0 ? [] : _d;
            _this._createFolder();
            var LOG_NAME = "".concat(_this.LOG_FOLDER, "/").concat(_this.LOG_NAME);
            var data;
            if (type === 'error') {
                var stackTrace = stack.map(function (data, index) { return "#".concat(index, " ").concat(data.trim(), "\n"); });
                stackTrace.shift();
                data = "".concat(_this._dateString(), " ").concat(type, ": ").concat(message, " \nStack trace: \n").concat(stackTrace.join(""));
            }
            else
                data = "".concat(_this._dateString(), " ").concat(type, ": ").concat(message);
            try {
                if (!fs_1.default.existsSync(LOG_NAME))
                    fs_1.default.writeFileSync(LOG_NAME, data);
                else {
                    var old = fs_1.default.readFileSync(LOG_NAME, 'utf8');
                    var newData = old ? "".concat(old, "\n").concat(data) : data;
                    fs_1.default.writeFileSync(LOG_NAME, newData);
                }
            }
            catch (err) {
                console.log("can't push file logs");
            }
            return _this;
        };
        this._findStack = function (err) {
            try {
                if (err.stack == null)
                    return [];
                var stackData = err.stack.split("\n");
                var stack = stackData !== null && stackData !== void 0 ? stackData : [];
                return stack;
            }
            catch (err) {
                return [];
            }
        };
        this.LOG_NAME = "".concat(this._dateString({ time: false }), ".log");
        this.LOG_FOLDER = path_1.default.resolve("logs");
    }
    Logger.prototype.createLogger = function (name) {
        if (name === void 0) { name = 'logs'; }
        this.LOG_FOLDER = path_1.default.resolve(name);
        return this;
    };
    Logger.prototype.readLogs = function (name) {
        if (name === void 0) { name = null; }
        var file = name ? "".concat(this.LOG_FOLDER, "/").concat(name) : "".concat(this.LOG_FOLDER, "/").concat(this.LOG_NAME);
        var rawLogs = fs_1.default.readFileSync(file, 'utf8');
        var logs = rawLogs === null || rawLogs === void 0 ? void 0 : rawLogs.split('\n');
        return logs !== null && logs !== void 0 ? logs : [];
    };
    Logger.prototype.readFolder = function (name) {
        var _a;
        if (name === void 0) { name = null; }
        try {
            var file = name ? "".concat(name) : "".concat(this.LOG_FOLDER);
            return (_a = fs_1.default.readdirSync(file)) !== null && _a !== void 0 ? _a : [];
        }
        catch (err) {
            console.log(err);
        }
    };
    Logger.prototype.error = function (err) {
        var stack = this._findStack(err);
        this._createLogFiles({
            type: this.LOG_TYPE.ERROR,
            message: err.message || 'unknow message',
            stack: stack
        });
        return;
    };
    Logger.prototype.info = function (message) {
        this._createLogFiles({
            type: this.LOG_TYPE.INFO,
            message: message,
            stack: []
        });
        return;
    };
    Logger.prototype.warn = function (message) {
        this._createLogFiles({
            type: this.LOG_TYPE.WARN,
            message: message,
            stack: []
        });
        return;
    };
    Logger.prototype.debug = function (message) {
        this._createLogFiles({
            type: this.LOG_TYPE.DEBUG,
            message: message,
            stack: []
        });
        return;
    };
    return Logger;
}());
exports.Logger = Logger;
