"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Validate = void 0;
var RULE_TYPE;
(function (RULE_TYPE) {
    RULE_TYPE["STRING"] = "string";
    RULE_TYPE["ARRAY"] = "array";
    RULE_TYPE["ARRAY_OBJECT"] = "array object";
    RULE_TYPE["OBJECT"] = "object";
    RULE_TYPE["BOOLEAN"] = "boolean";
    RULE_TYPE["COMFIRM"] = "confirm";
    RULE_TYPE["NUMBER"] = "number";
    RULE_TYPE["DATE"] = "date";
    RULE_TYPE["MAX"] = "max";
    RULE_TYPE["MIN"] = "min";
    RULE_TYPE["EMAIL"] = "email";
    RULE_TYPE["REQUIRED"] = "required";
    RULE_TYPE["REGEX"] = "regex";
})(RULE_TYPE || (RULE_TYPE = {}));
var Rule = /** @class */ (function () {
    function Rule() {
        this.RULES = [];
        this.MESSAGES = [];
    }
    /**
     * required value. value is not empty
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.required = function (message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.REQUIRED], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.number = function (message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.NUMBER], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.string = function (message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.STRING], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.array = function (message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.ARRAY], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.email = function (message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.EMAIL], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.date = function (message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.DATE], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {string} regexRule regex
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.regex = function (regexRule, message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), ["".concat(RULE_TYPE.REGEX, ":").concat(regexRule)], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {string} key key name for compare
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.confirm = function (key, message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), ["".concat(RULE_TYPE.COMFIRM, ":").concat(key)], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {number} limit limit characters
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.max = function (limit, message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), ["".concat(RULE_TYPE.MAX, ":").concat(limit)], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {number} limit limit characters
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.min = function (limit, message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), ["".concat(RULE_TYPE.MIN, ":").concat(limit)], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     *
     * @param {string} message  error message
     * @return {this} this
     */
    Rule.prototype.boolean = function (message) {
        if (message === void 0) { message = ''; }
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.BOOLEAN], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        return this;
    };
    /**
     * if has 1 arguments. default message = callback
     * @param {string} message
     * @param {function | undefined} cb callback Rule for validate values
     * @return {this} this
     */
    Rule.prototype.object = function (message, cb) {
        if (arguments.length === 1) {
            cb = message;
            message = '';
        }
        if (typeof cb !== 'function')
            throw new Error('Invalid callback');
        var rule = cb(Rule);
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.OBJECT], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        this.NEST_RULES = __assign(__assign({}, this.NEST_RULES), rule);
        return this;
    };
    /**
     * if has 1 arguments. default message = callback
     * @param {string} message
     * @param {function | undefined} cb callback Rule for validate values
     * @return {this} this
     */
    Rule.prototype.arrayObject = function (message, cb) {
        if (arguments.length === 1) {
            cb = message;
            message = '';
        }
        if (typeof cb !== 'function')
            throw new Error('Invalid callback');
        var rule = cb(Rule);
        this.RULES = __spreadArray(__spreadArray([], __read(this.RULES), false), [RULE_TYPE.ARRAY_OBJECT], false);
        this.MESSAGES = __spreadArray(__spreadArray([], __read(this.MESSAGES), false), [message], false);
        this.NEST_RULES = __assign(__assign({}, this.NEST_RULES), rule);
        return this;
    };
    return Rule;
}());
var Validate = /** @class */ (function () {
    function Validate(data) {
        var _this = this;
        this.ERRORS = {};
        this.REQUESTS = {};
        /**
         *
         * check data is correct in the rule validation
         * @param {Rule} callback callback Rule for validation
         * @return {this} this
         */
        this.check = function (callback) {
            var dataRules = callback(Rule);
            if (!_this._typeObject(dataRules) || !_this.REQUESTS)
                throw new Error('invalid rules');
            return _this._validate(dataRules);
        };
        /**
         *
         * @return {boolean} boolean
         */
        this.fails = function () { var _a, _b; return (_b = !!((_a = Object.keys(_this.ERRORS)) === null || _a === void 0 ? void 0 : _a.length)) !== null && _b !== void 0 ? _b : true; };
        /**
         *
         * @return {object} object
         */
        this.errors = function () { return _this.ERRORS; };
        /**
         *
         * @return {string} string
         */
        this.errorsToString = function () { return JSON.stringify(_this.ERRORS, null, 2); };
        this._type = function (data) { return Object.prototype.toString.apply(data).slice(8, -1); };
        this._typeObject = function (data) { return _this._type(data) === 'Object'; };
        this._isEmpty = function (value) {
            var valueIsType = (Object.prototype.toString.apply(value).slice(8, -1)).toLowerCase();
            switch (valueIsType) {
                case 'array': {
                    if (!value.length)
                        return true;
                    break;
                }
                case 'object': {
                    if (!Object.keys(value).length)
                        return true;
                    break;
                }
                case 'string': {
                    if (value === '')
                        return true;
                    break;
                }
                case 'null':
                case 'undefined': {
                    if (value == null)
                        return true;
                    break;
                }
                default: {
                    throw new Error("unknown type of value");
                }
            }
            return false;
        };
        this._object = function (rules, data, message, cb) {
            var _a;
            if (_this._type(data) !== 'Object') {
                cb([message === '' ? 'object only' : message]);
                return;
            }
            var validate = new Validate(data);
            validate._validate(rules);
            if ((_a = Object.keys(validate.errors())) === null || _a === void 0 ? void 0 : _a.length)
                cb(validate.errors());
            return;
        };
        this._arrayObject = function (rules, data, message, cb) {
            var _a;
            if (_this._type(data) !== 'Array') {
                cb([message === '' ? 'Array only' : message]);
                return;
            }
            for (var index in data) {
                var element = data[index];
                var validate = new Validate(element);
                validate._validate(rules);
                if ((_a = Object.keys(validate.errors())) === null || _a === void 0 ? void 0 : _a.length) {
                    cb(__assign({ index: Number(index) }, validate.errors()));
                }
            }
            return;
        };
        this._boolean = function (reqInput, message, cb) {
            if (_this._isEmpty(message))
                message = "boolean only";
            if (typeof reqInput !== "boolean")
                cb(message);
            return;
        };
        this._required = function (reqInput, message, cb) {
            if (_this._isEmpty(message))
                message = "required";
            if (_this._isEmpty(reqInput))
                cb(message);
            return;
        };
        this._string = function (reqInput, message, cb) {
            var regex = /^[ก-๏\s-a-zA-Z]+$/;
            if (_this._isEmpty(message))
                message = "string only";
            if (!regex.test(reqInput))
                cb(message);
            return;
        };
        this._array = function (reqInput, message, cb) {
            if (_this._isEmpty(message))
                message = "array only";
            if (!Array.isArray(reqInput))
                cb(message);
            return;
        };
        this._date = function (reqInput, message, cb) {
            var regex = /\d{4}-\d{2}-\d{2}/;
            if (_this._isEmpty(message))
                message = "must be valid ( YYYY-MM-DD)!";
            if (!regex.test(reqInput))
                cb(message);
            return;
        };
        this._email = function (reqInput, message, cb) {
            var regex = /\S+@\S+\.\S+/;
            if (_this._isEmpty(message))
                message = "email must be invalid";
            if (!regex.test(reqInput))
                cb(message);
            return;
        };
        this._number = function (reqInput, message, cb) {
            var regex = /^\d+$/;
            if (_this._isEmpty(message))
                message = "number only";
            if (!regex.test(reqInput))
                cb(message);
            return;
        };
        this._regex = function (reqInput, rawRegex, message, cb) {
            var regexRule = new RegExp(rawRegex);
            if (_this._isEmpty(message))
                message = "must not be regex ".concat(rawRegex, " in characters");
            if (regexRule.test(reqInput))
                cb(message);
            return;
        };
        this._max = function (reqInput, max, message, cb) {
            if (_this._isEmpty(message))
                message = "maximum ".concat(max, " characters");
            if (reqInput && reqInput.length > max)
                cb(message);
            return;
        };
        this._min = function (reqInput, min, message, cb) {
            if (_this._isEmpty(message))
                message = "minimum ".concat(min, " characters");
            if (reqInput && reqInput.length < min)
                cb(message);
            return;
        };
        this._confirm = function (data, reqInput, key, message, cb) {
            var compare = reqInput === data[key];
            if (_this._isEmpty(message))
                message = "not match";
            if (!compare)
                cb(message);
            return;
        };
        this.REQUESTS = data;
    }
    Validate.prototype._validate = function (dataRules) {
        var e_1, _a;
        var _this = this;
        var _b, _c;
        var requests = this.REQUESTS;
        var errorValidate = {};
        var _loop_1 = function (property, rawRules) {
            var reqInput = requests[property];
            if (!rawRules)
                throw new Error('invalid rules');
            var _g = dataRules[property], RULES = _g.RULES, MESSAGES = _g.MESSAGES, NEST_RULES = _g.NEST_RULES;
            for (var index in RULES) {
                var message = MESSAGES[index];
                var rule = RULES[index];
                if (rule === RULE_TYPE.STRING) {
                    this_1._string(reqInput, message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === RULE_TYPE.EMAIL) {
                    this_1._email(reqInput, message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === RULE_TYPE.NUMBER) {
                    this_1._number(reqInput, message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === RULE_TYPE.DATE) {
                    this_1._date(reqInput, message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === RULE_TYPE.BOOLEAN) {
                    this_1._boolean(reqInput, message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === null || rule === void 0 ? void 0 : rule.includes(RULE_TYPE.COMFIRM)) {
                    this_1._confirm(requests, reqInput, (_b = rule === null || rule === void 0 ? void 0 : rule.split(':')) === null || _b === void 0 ? void 0 : _b.pop(), message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                }
                if (rule.includes(RULE_TYPE.REGEX)) {
                    this_1._regex(reqInput, rule === null || rule === void 0 ? void 0 : rule.split(':').pop(), message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === null || rule === void 0 ? void 0 : rule.includes(RULE_TYPE.MAX)) {
                    this_1._max(reqInput, rule === null || rule === void 0 ? void 0 : rule.split(':').pop(), message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule.includes(RULE_TYPE.MIN)) {
                    this_1._min(reqInput, rule.split(':').pop(), message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === RULE_TYPE.ARRAY) {
                    this_1._array(reqInput, message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === RULE_TYPE.OBJECT) {
                    this_1._object(NEST_RULES, reqInput, message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidateObject(errorValidate, property, err);
                    });
                    continue;
                }
                if (rule === RULE_TYPE.ARRAY_OBJECT) {
                    this_1._arrayObject(NEST_RULES, reqInput, message, function (err) {
                        if (err)
                            errorValidate = _this._errorValidate(errorValidate, property, err);
                    });
                    continue;
                }
                this_1._required(reqInput, message, function (err) {
                    if (err)
                        errorValidate = _this._errorValidate(errorValidate, property, err);
                });
            }
        };
        var this_1 = this;
        try {
            for (var _d = __values(Object.entries(dataRules)), _e = _d.next(); !_e.done; _e = _d.next()) {
                var _f = __read(_e.value, 2), property = _f[0], rawRules = _f[1];
                _loop_1(property, rawRules);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d.return)) _a.call(_d);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if ((_c = Object.keys(errorValidate)) === null || _c === void 0 ? void 0 : _c.length)
            this.ERRORS = errorValidate;
        return this;
    };
    Validate.prototype._errorValidate = function (errors, property, err) {
        var _a;
        return errors = __assign(__assign({}, errors), (_a = {}, _a[property] = errors.hasOwnProperty(property) ? __spreadArray(__spreadArray([], __read(errors[property]), false), [err], false) : [err], _a));
    };
    Validate.prototype._errorValidateObject = function (errors, property, err) {
        var _a;
        return errors = __assign(__assign({}, errors), (_a = {}, _a[property] = err, _a));
    };
    return Validate;
}());
exports.Validate = Validate;
